@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2021 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2025 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ extern mode_t saved_umask;
 extern char *prog_name;
 extern uid_t rootuid;
 extern gid_t rootgid;
-extern char sig_hup;
+extern bool sig_hup;
 extern struct cf_t *file_base;
 extern struct job_t *queue_base;
 extern unsigned long int next_id;
