@@ -1,7 +1,7 @@
 /*
- * FCRON - periodic command scheduler 
+ * FCRON - periodic command scheduler
  *
- *  Copyright 2000-2021 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2025 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -12,11 +12,11 @@
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- * 
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
+ *
  *  The GNU General Public License can also be found in the file
  *  `LICENSE' that comes with the fcron source distribution.
  */
@@ -58,6 +58,7 @@ int fcrondyn_cl_num = 0;        /* number of fcrondyn clients currently connecte
 int listen_fd = -1;             /* fd which catches incoming connection */
 int auth_fail = 0;              /* number of auth failure */
 time_t auth_nofail_since = 0;   /* we refuse auth since x due to too many failures */
+static char truncated[] = " (truncated)\n";
 
 /* some error messages ... */
 char err_no_err_str[] = "Command successfully completed.\n";
@@ -104,7 +105,7 @@ char err_others_nallowed_str[] =
 
 
 void
-fcrondyn_socket_init(select_instance * si)
+fcrondyn_socket_init(select_instance *si)
     /* do everything needed to get a working listening socket */
 {
     struct sockaddr_un addr;
@@ -250,7 +251,7 @@ auth_client_so_peercred(struct fcrondyn_cl *client)
      * Sets client->fcl_user on success, don't do anything on failure
      * so that the client stays unauthenticated */
 {
-    const int true = 1;
+    const int true_val = 1;
     /* There is no ucred.h (or equivalent) on linux to define struct ucred (!!)
      * so we do it here */
 #if ! ( defined(HAVE_CRED_H) && defined(HAVE_UCRED_H) \
@@ -265,8 +266,8 @@ auth_client_so_peercred(struct fcrondyn_cl *client)
     socklen_t cred_size = sizeof(cred);
     struct passwd *p_entry = NULL;
 
-    setsockopt(client->fcl_sock_fd, SOL_SOCKET, SO_PASSCRED, &true,
-               sizeof(true));
+    setsockopt(client->fcl_sock_fd, SOL_SOCKET, SO_PASSCRED, &true_val,
+               sizeof(true_val));
     if (getsockopt
         (client->fcl_sock_fd, SOL_SOCKET, SO_PEERCRED, &cred,
          &cred_size) != 0) {
@@ -362,7 +363,7 @@ auth_client_password(struct fcrondyn_cl *client)
 
 #define Test_add_field(FIELD_NB, FIELD_STR) \
     if ( (bit_test(details, FIELD_NB)) ) { \
-        strncat(fields, FIELD_STR, sizeof(fields)-1 - len); \
+        strncat(fields, FIELD_STR, sizeof(fields) - len -1); \
         len += (sizeof(FIELD_STR)-1); \
     }
 #define Add_field(FIELD_STR) \
@@ -378,7 +379,7 @@ print_fields(int fd, unsigned char *details)
     char field_user[] = "|USER     ";
     char field_rq[] = "|R&Q ";
     char field_options[] = "|OPTIONS  ";
-    char field_schedule[] = "|SCHEDULE        ";
+    char field_schedule[] = "|SCHEDULE           ";
     char field_until[] = "|LAVG 1,5,15 UNTIL       STRICT";
     char field_pid[] = "|PID    ";
     char field_index[] = "|INDEX";
@@ -399,9 +400,13 @@ print_fields(int fd, unsigned char *details)
     Add_field(field_cmd);
     Add_field(field_endline);
 
-    fields[TERM_LEN - 1] = '\0';
+    /* Extra safety (which should be redundant as strncat (used in Add_field
+     * and Test_add_field) always null-terminate the string: */
+    fields[sizeof(fields) - 1] = '\0';
 
-    if (send(fd, fields, (len < sizeof(fields)) ? len : sizeof(fields), 0) < 0)
+    /* add +1 to include the final end-of-string "\0" */
+    if (send(fd, fields, (len + 1 < sizeof(fields)) ? len + 1 : sizeof(fields),
+             0) < 0)
         error_e("error in send()");
 
 }
@@ -452,9 +457,10 @@ print_line(int fd, struct cl_t *line, unsigned char *details, pid_t pid,
             ftime = localtime(&until);
             len +=
                 snprintf(buf + len, sizeof(buf) - len,
-                         " %04d-%02d-%02d %02d:%02d %s",
+                         " %04d-%02d-%02d %02d:%02d:%02d %s",
                          (ftime->tm_year + 1900), (ftime->tm_mon + 1),
                          ftime->tm_mday, ftime->tm_hour, ftime->tm_min,
+                         ftime->tm_sec,
                          (is_strict(line->cl_option)) ? "Y" : "N");
         }
         else
@@ -465,15 +471,28 @@ print_line(int fd, struct cl_t *line, unsigned char *details, pid_t pid,
     if (bit_test(details, FIELD_SCHEDULE)) {
         ftime = localtime(&(line->cl_nextexe));
         len +=
-            snprintf(buf + len, sizeof(buf) - len, "|%04d-%02d-%02d %02d:%02d",
-                     (ftime->tm_year + 1900), (ftime->tm_mon + 1),
-                     ftime->tm_mday, ftime->tm_hour, ftime->tm_min);
+            snprintf(buf + len, sizeof(buf) - len,
+                     "|%04d-%02d-%02d %02d:%02d:%02d", (ftime->tm_year + 1900),
+                     (ftime->tm_mon + 1), ftime->tm_mday, ftime->tm_hour,
+                     ftime->tm_min, ftime->tm_sec);
     }
     len += snprintf(buf + len, sizeof(buf) - len, "|%s\n", line->cl_shell);
 
-    if (send(fd, buf, (len < sizeof(buf)) ? len : sizeof(buf), 0) < 0)
-        error_e("error in send()");
+    /* snprintf() returns the length of what would have been written (excluding terminating null byte)
+     * if not limited by maxlen */
+    if (len + 1 > sizeof(buf)) {
+        /* the shell command string was too long and was truncated */
+        strcpy(buf + sizeof(buf) - sizeof(truncated), truncated);
+    }
 
+    /* as extra safety to make sure the string is always null-terminated
+     * (even though snprintf man page suggests it does it already) */
+    buf[sizeof(buf) - 1] = '\0';
+
+    /* add +1 to include the final end-of-string "\0" */
+    if (send(fd, buf, (len + 1 < sizeof(buf)) ? len + 1 : sizeof(buf), 0) < 0) {
+        error_e("error in send()");
+    }
 }
 
 
@@ -530,7 +549,8 @@ cmd_ls(struct fcrondyn_cl *client, long int *cmd, int fd, int is_root)
             getloadavg(lavg, 3);
             i = snprintf(lavg_str, sizeof(lavg_str), "Current load average : "
                          "%.1f, %.1f, %.1f\n", lavg[0], lavg[1], lavg[2]);
-            send(fd, lavg_str, i, 0);
+            send(fd, lavg_str,
+                 (i + 1 < sizeof(lavg_str)) ? i + 1 : sizeof(lavg_str), 0);
 
             bit_set(fields, FIELD_LAVG);
         }
@@ -589,7 +609,7 @@ cmd_ls(struct fcrondyn_cl *client, long int *cmd, int fd, int is_root)
                  e = exe_list_next(exe_list)) {
                 if (e->e_line == NULL) {
                     if (is_root) {
-                        send_msg_fd(fd, "job no more in an fcrontab: pid %d",
+                        send_msg_fd(fd, "Job no more in an fcrontab: pid %d",
                                     e->e_job_pid);
                         found = 1;
                     }
@@ -702,7 +722,7 @@ cmd_on_exeq(struct fcrondyn_cl *client, long int *cmd, int fd, int is_root)
 
 
 void
-cmd_renice(struct fcrondyn_cl *client, long int *cmd, int fd, exe_t * e,
+cmd_renice(struct fcrondyn_cl *client, long int *cmd, int fd, exe_t *e,
            int is_root)
 /* change nice value of a running job */
 {
@@ -739,7 +759,7 @@ cmd_renice(struct fcrondyn_cl *client, long int *cmd, int fd, exe_t * e,
 
 
 void
-cmd_send_signal(struct fcrondyn_cl *client, long int *cmd, int fd, exe_t * e)
+cmd_send_signal(struct fcrondyn_cl *client, long int *cmd, int fd, exe_t *e)
 /* send a signal to a running job */
 {
     if (e->e_job_pid <= 0 || (int)cmd[1] <= 0) {
@@ -843,8 +863,8 @@ exe_cmd(struct fcrondyn_cl *client)
 
 void
 remove_connection(struct fcrondyn_cl **client, struct fcrondyn_cl *prev_client,
-                  select_instance * si)
-/* close the connection, remove it from the list 
+                  select_instance *si)
+/* close the connection, remove it from the list
 and make client points to the next entry */
 {
     debug("closing connection : fd : %d", (*client)->fcl_sock_fd);
@@ -867,7 +887,7 @@ and make client points to the next entry */
 }
 
 void
-fcrondyn_socket_check(select_instance * si)
+fcrondyn_socket_check(select_instance *si)
     /* check for new connection, command, connection closed */
 {
     int fd = -1, avoid_fd = -1;
@@ -985,7 +1005,7 @@ fcrondyn_socket_check(select_instance * si)
 
 
 void
-fcrondyn_socket_close(select_instance * si)
+fcrondyn_socket_close(select_instance *si)
     /* close connections, close socket, remove socket file.
      * If si is not NULL, then remove the fds from si's readfds */
 {
